<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

### Table of Contents

-   [DataFrame](#dataframe)
    -   [constructor](#constructor)
    -   [toDict](#todict)
    -   [toArray](#toarray)
    -   [toCollection](#tocollection)
    -   [toDSV](#todsv)
    -   [toText](#totext)
    -   [toCSV](#tocsv)
    -   [toTSV](#totsv)
    -   [toPSV](#topsv)
    -   [toJSON](#tojson)
    -   [show](#show)
    -   [dim](#dim)
    -   [transpose](#transpose)
    -   [count](#count)
    -   [countValue](#countvalue)
    -   [push](#push)
    -   [replace](#replace)
    -   [distinct](#distinct)
    -   [unique](#unique)
    -   [listColumns](#listcolumns)
    -   [select](#select)
    -   [withColumn](#withcolumn)
    -   [restructure](#restructure)
    -   [renameAll](#renameall)
    -   [rename](#rename)
    -   [castAll](#castall)
    -   [cast](#cast)
    -   [drop](#drop)
    -   [chain](#chain)
    -   [filter](#filter)
    -   [where](#where)
    -   [find](#find)
    -   [map](#map)
    -   [reduce](#reduce)
    -   [reduceRight](#reduceright)
    -   [dropDuplicates](#dropduplicates)
    -   [shuffle](#shuffle)
    -   [sample](#sample)
    -   [bisect](#bisect)
    -   [groupBy](#groupby)
    -   [sortBy](#sortby)
    -   [union](#union)
    -   [join](#join)
    -   [innerJoin](#innerjoin)
    -   [fullJoin](#fulljoin)
    -   [outerJoin](#outerjoin)
    -   [leftJoin](#leftjoin)
    -   [rightJoin](#rightjoin)
    -   [diff](#diff)
    -   [setDefaultModules](#setdefaultmodules)
    -   [fromDSV](#fromdsv)
    -   [fromText](#fromtext)
    -   [fromCSV](#fromcsv)
    -   [fromTSV](#fromtsv)
    -   [fromPSV](#frompsv)
    -   [fromJSON](#fromjson)

## DataFrame

[src/dataframe.js:15-986](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L15-L986 "Source code on GitHub")

DataFrame data structure providing an immutable, flexible and powerfull way to manipulate data with columns and rows.

**Parameters**

-   `data`  
-   `columns`  
-   `modules` **...any** 

### constructor

[src/dataframe.js:174-184](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L174-L184 "Source code on GitHub")

Create a new DataFrame.

**Parameters**

-   `data` **([Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array) \| [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object) \| [DataFrame](#dataframe))** The data of the DataFrame.
-   `columns` **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** The DataFrame column names.
-   `modules` **...[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)?** Additional modules.

**Examples**

```javascript
new DataFrame({
     'column1': [3, 6, 8],
     'column2': [3, 4, 5, 6],
}, ['column1', 'column2'])

new Data Frame([
     [1, 6, 9, 10, 12],
     [1, 2],
     [6, 6, 9, 8, 9, 12],
], ['c1', 'c2', 'c3', 'c4', 'c5', 'c6'])

new DataFrame([
     {c1: 1, c2: 6, c3: 9, c4: 10, c5: 12},
     {c4: 1, c3: 2},
     {c1: 6, c5: 6, c2: 9, c4: 8, c3: 9, c6: 12},
], ['c1', 'c2', 'c3', 'c4', 'c5', 'c6'])

new DataFrame(df);
```

### toDict

[src/dataframe.js:288-292](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L288-L292 "Source code on GitHub")

Convert DataFrame into dict / hash / object.

**Examples**

```javascript
df.toDict()
```

Returns **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** The DataFrame converted into dict.

### toArray

[src/dataframe.js:301-303](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L301-L303 "Source code on GitHub")

Convert DataFrame into Array of Arrays. You can also extract only one column as Array.

**Parameters**

-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** Column Name to extract. By default, all columns are transformed.

**Examples**

```javascript
df.toArray()
```

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** The DataFrame (or the column) converted into Array.

### toCollection

[src/dataframe.js:312-314](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L312-L314 "Source code on GitHub")

Convert DataFrame into Array of dictionnaries. You can also return Rows instead of dictionnaries.

**Parameters**

-   `ofRows` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)?** Return a collection of Rows instead of dictionnaries.

**Examples**

```javascript
df.toCollection()
```

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** The DataFrame converted into Array of dictionnaries (or Rows).

### toDSV

[src/dataframe.js:330-337](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L330-L337 "Source code on GitHub")

Convert the DataFrame into a text delimiter separated values.
You can also save the file if you are using nodejs.

**Parameters**

-   `sep` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** Column separator. (optional, default `' '`)
-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Writing the header in the first line. If false, there will be no header. (optional, default `true`)
-   `path` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** The path to save the file. /!\\ Works only on node.js, not into the browser. (optional, default `undefined`)

**Examples**

```javascript
df.toDSV()
df.toDSV(';')
df.toDSV(';', true)
// From node.js only
df.toDSV(';', true, '/my/absolute/path/dataframe.txt')
```

Returns **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The text file in raw string.

### toText

[src/dataframe.js:353-355](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L353-L355 "Source code on GitHub")

Convert the DataFrame into a text delimiter separated values. Alias for .toDSV.
You can also save the file if you are using nodejs.

**Parameters**

-   `sep` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** Column separator. (optional, default `' '`)
-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Writing the header in the first line. If false, there will be no header. (optional, default `true`)
-   `path` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** The path to save the file. /!\\ Works only on node.js, not into the browser. (optional, default `undefined`)

**Examples**

```javascript
df.toText()
df.toText(';')
df.toText(';', true)
// From node.js only
df.toText(';', true, '/my/absolute/path/dataframe.txt')
```

Returns **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The text file in raw string.

### toCSV

[src/dataframe.js:369-371](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L369-L371 "Source code on GitHub")

Convert the DataFrame into a comma separated values string.
You can also save the file if you are using nodejs.

**Parameters**

-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Writing the header in the first line. If false, there will be no header. (optional, default `true`)
-   `path` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** The path to save the file. /!\\ Works only on node.js, not into the browser. (optional, default `undefined`)

**Examples**

```javascript
df.toCSV()
df.toCSV(true)
// From node.js only
df.toCSV(true, '/my/absolute/path/dataframe.csv')
```

Returns **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The csv file in raw string.

### toTSV

[src/dataframe.js:385-387](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L385-L387 "Source code on GitHub")

Convert the DataFrame into a tab separated values string.
You can also save the file if you are using nodejs.

**Parameters**

-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Writing the header in the first line. If false, there will be no header. (optional, default `true`)
-   `path` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** The path to save the file. /!\\ Works only on node.js, not into the browser. (optional, default `undefined`)

**Examples**

```javascript
df.toCSV()
df.toCSV(true)
// From node.js only
df.toCSV(true, '/my/absolute/path/dataframe.csv')
```

Returns **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The csv file in raw string.

### toPSV

[src/dataframe.js:401-403](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L401-L403 "Source code on GitHub")

Convert the DataFrame into a pipe separated values string.
You can also save the file if you are using nodejs.

**Parameters**

-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Writing the header in the first line. If false, there will be no header. (optional, default `true`)
-   `path` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** The path to save the file. /!\\ Works only on node.js, not into the browser. (optional, default `undefined`)

**Examples**

```javascript
df.toPSV()
df.toPSV(true)
// From node.js only
df.toPSV(true, '/my/absolute/path/dataframe.csv')
```

Returns **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The csv file in raw string.

### toJSON

[src/dataframe.js:415-419](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L415-L419 "Source code on GitHub")

Convert the DataFrame into a json string. You can also save the file if you are using nodejs.

**Parameters**

-   `asCollection` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Writing the JSON as collection of Object. (optional, default `true`)
-   `path` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** The path to save the file. /!\\ Works only on node.js, not into the browser. (optional, default `undefined`)

**Examples**

```javascript
df.toJSON()
// From node.js only
df.toJSON('/my/absolute/path/dataframe.json')
```

Returns **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The json file in raw string.

### show

[src/dataframe.js:431-450](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L431-L450 "Source code on GitHub")

Display the DataFrame as String Table. Can only return a sring instead of displaying the DataFrame.

**Parameters**

-   `rows` **[Number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** The number of lines to display. (optional, default `10`)
-   `quiet` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Quiet mode. If true, only returns a string instead of console.log(). (optional, default `false`)

**Examples**

```javascript
df.show()
df.show(10)
const stringDF = df.show(10, true)
```

Returns **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The DataFrame as String Table.

### dim

[src/dataframe.js:458-460](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L458-L460 "Source code on GitHub")

Get the DataFrame dimensions.

**Examples**

```javascript
const [height, weight] = df.dim()
```

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** The DataFrame dimensions. [height, weight]

### transpose

[src/dataframe.js:469-474](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L469-L474 "Source code on GitHub")

Transpose a DataFrame. Rows become columns and conversely. n x p => p x n.

**Parameters**

-   `tranposeColumnNames`  
-   `transposeColumnNames` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** An option to transpose columnNames in a rowNames column. (optional, default `false`)

**Examples**

```javascript
df.transpose()
```

Returns **ÐataFrame** A new transposed DataFrame.

### count

[src/dataframe.js:482-484](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L482-L484 "Source code on GitHub")

Get the rows number.

**Examples**

```javascript
df.count()
```

Returns **Int** The number of DataFrame rows.

### countValue

[src/dataframe.js:495-497](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L495-L497 "Source code on GitHub")

Get the count of a value into a column.

**Parameters**

-   `valueToCount`  The value to count into the selected column.
-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column to count the value. (optional, default `this.listColumns()[0]`)

**Examples**

```javascript
df.countValue(5, 'column2')
df.select('column1').countValue(5)
```

Returns **Int** The number of times the selected value appears.

### push

[src/dataframe.js:506-508](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L506-L508 "Source code on GitHub")

Push new rows into the DataFrame.

**Parameters**

-   `rows` **([Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array) | Row)** The rows to add.

**Examples**

```javascript
df.push([1,2,3], [1,4,9])
```

Returns **[DataFrame](#dataframe)** A new DataFrame with the new rows.

### replace

[src/dataframe.js:519-524](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L519-L524 "Source code on GitHub")

Replace a value by another in all the DataFrame or in a column.

**Parameters**

-   `value`  The value to replace.
-   `replacement`  The new value.
-   `columnNames` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array))** The columns to apply the replacement. (optional, default `this.listColumns()`)

**Examples**

```javascript
df.replace(undefined, 0, 'column1', 'column2')
```

Returns **[DataFrame](#dataframe)** A new DataFrame with replaced values.

### distinct

[src/dataframe.js:533-537](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L533-L537 "Source code on GitHub")

Compute unique values into a column.

**Parameters**

-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column to distinct.

**Examples**

```javascript
df.distinct('column1')
```

Returns **[DataFrame](#dataframe)** A DataFrame containing the column with distinct values.

### unique

[src/dataframe.js:547-549](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L547-L549 "Source code on GitHub")

Compute unique values into a column.
Alias from .distinct()

**Parameters**

-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column to distinct.

**Examples**

```javascript
df.unique('column1')
```

Returns **[DataFrame](#dataframe)** A DataFrame containing the column with distinct values.

### listColumns

[src/dataframe.js:557-559](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L557-L559 "Source code on GitHub")

List DataFrame columns.

**Examples**

```javascript
df.listColumns()
```

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** An Array containing DataFrame columnNames.

### select

[src/dataframe.js:568-572](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L568-L572 "Source code on GitHub")

Select columns in the DataFrame.

**Parameters**

-   `columnNames` **...[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The columns to select.

**Examples**

```javascript
df.select('column1', 'column3')
```

Returns **[DataFrame](#dataframe)** A new DataFrame containing selected columns.

### withColumn

[src/dataframe.js:583-589](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L583-L589 "Source code on GitHub")

Add a new column or set an existing one.

**Parameters**

-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column to modify or to create.
-   `func` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** The function to create the column. (optional, default `(row,index)`)

**Examples**

```javascript
df.withColumn('column4', () => 2)
df.withColumn('column2', (row) => row.get('column2') * 2)
```

Returns **[DataFrame](#dataframe)** A new DataFrame containing the new or modified column.

### restructure

[src/dataframe.js:600-602](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L600-L602 "Source code on GitHub")

Modify the structure of the DataFrame by changing columns order, creating new columns or removing some columns.

**Parameters**

-   `newColumnNames` **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** The new columns of the DataFrame.

**Examples**

```javascript
df.restructure(['column1', 'column4', 'column2', 'column3'])
df.restructure(['column1', 'column4'])
df.restructure(['column1', 'newColumn', 'column4'])
```

Returns **[DataFrame](#dataframe)** A new DataFrame with restructured columns (renamed, add or deleted).

### renameAll

[src/dataframe.js:611-616](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L611-L616 "Source code on GitHub")

Rename each column.

**Parameters**

-   `newColumnNames` **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** The new column names of the DataFrame.

**Examples**

```javascript
df.renameAll(['column1', 'column3', 'column4'])
```

Returns **[DataFrame](#dataframe)** A new DataFrame with the new column names.

### rename

[src/dataframe.js:626-629](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L626-L629 "Source code on GitHub")

Rename a column.

**Parameters**

-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column to rename.
-   `replacement` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The new name for the column.

**Examples**

```javascript
df.rename('column1', 'columnRenamed')
```

Returns **[DataFrame](#dataframe)** A new DataFrame with the new column name.

### castAll

[src/dataframe.js:638-645](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L638-L645 "Source code on GitHub")

Cast each column into a given type.

**Parameters**

-   `typeFunctions` **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** The functions used to cast columns.

**Examples**

```javascript
df.castAll([Number, String, (val) => new CustomClass(val)])
```

Returns **[DataFrame](#dataframe)** A new DataFrame with the columns having new types.

### cast

[src/dataframe.js:656-658](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L656-L658 "Source code on GitHub")

Cast a column into a given type.

**Parameters**

-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column to cast.
-   `typeFunction`  
-   `ObjectType` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** The function used to cast the column.

**Examples**

```javascript
df.cast('column1', Number)
df.cast('column1', (val) => new MyCustomClass(val))
```

Returns **[DataFrame](#dataframe)** A new DataFrame with the column having a new type.

### drop

[src/dataframe.js:667-671](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L667-L671 "Source code on GitHub")

Remove a single column.

**Parameters**

-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column to drop.

**Examples**

```javascript
df.drop('column2')
```

Returns **[DataFrame](#dataframe)** A new DataFrame without the dropped column.

### chain

[src/dataframe.js:686-688](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L686-L688 "Source code on GitHub")

Chain maps and filters functions on DataFrame by optimizing their executions.
If a function returns boolean, it's a filter. Else it's a map.
It can be 10 - 100 x faster than standard chains of .map() and .filter().

**Parameters**

-   `funcs` **...[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** Functions to apply on the DataFrame rows taking the row as parameter.

**Examples**

```javascript
df.chain(
     row => row.get('column1') > 3, // filter
     row => row.set('column1', 3),  // map
     row => row.get('column2') === '5' // filter
)
```

Returns **[DataFrame](#dataframe)** A new DataFrame with modified rows.

### filter

[src/dataframe.js:698-704](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L698-L704 "Source code on GitHub")

Filter DataFrame rows.

**Parameters**

-   `condition` **([Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function) \| [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object))** A filter function or a column/value object.

**Examples**

```javascript
df.filter(row => row.get('column1') >= 3)
df.filter({'column2': 5, 'column1': 3}))
```

Returns **[DataFrame](#dataframe)** A new filtered DataFrame.

### where

[src/dataframe.js:715-717](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L715-L717 "Source code on GitHub")

Filter DataFrame rows.
Alias of .filter()

**Parameters**

-   `condition` **([Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function) \| [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object))** A filter function or a column/value object.

**Examples**

```javascript
df.where(row => row.get('column1') >= 3)
df.where({'column2': 5, 'column1': 3}))
```

Returns **[DataFrame](#dataframe)** A new filtered DataFrame.

### find

[src/dataframe.js:727-729](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L727-L729 "Source code on GitHub")

Find a row (the first met) based on a condition.

**Parameters**

-   `condition` **([Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function) \| [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object))** A filter function or a column/value object.

**Examples**

```javascript
df.find(row => row.get('column1') === 3)
df.find({'column1': 3})
```

Returns **Row** The targeted Row.

### map

[src/dataframe.js:738-740](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L738-L740 "Source code on GitHub")

Map on DataFrame rows. /!\\ Prefer to use .chain().

**Parameters**

-   `func` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** A function to apply on each row taking the row as parameter.

**Examples**

```javascript
df.map(row => row.set('column1', row.get('column1') * 2))
```

Returns **[DataFrame](#dataframe)** A new DataFrame with modified rows.

### reduce

[src/dataframe.js:754-757](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L754-L757 "Source code on GitHub")

Reduce DataFrame into a value.

**Parameters**

-   `func` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** The reduce function taking 2 parameters, previous and next.
-   `init`  The initial value of the reducer.

**Examples**

```javascript
df.reduce((p, n) => n.get('column1') + p, 0)
df2.reduce((p, n) => (
         n.set('column1', p.get('column1') + n.get('column1'))
          .set('column2', p.get('column2') + n.get('column2'))
))
```

Returns **any** A reduced value.

### reduceRight

[src/dataframe.js:767-770](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L767-L770 "Source code on GitHub")

Reduce DataFrame into a value, starting from the last row (see .reduce()).

**Parameters**

-   `func` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** The reduce function taking 2 parameters, previous and next.
-   `init`  The initial value of the reducer.

**Examples**

```javascript
df.reduceRight((p, n) => p > n ? p : n, 0)
```

Returns **any** A reduced value.

### dropDuplicates

[src/dataframe.js:778-780](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L778-L780 "Source code on GitHub")

Return a DataFrame without duplicated columns.

**Examples**

```javascript
df.dropDuplicates()
```

Returns **[DataFrame](#dataframe)** A DataFrame without duplicated rows.

### shuffle

[src/dataframe.js:788-799](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L788-L799 "Source code on GitHub")

Return a shuffled DataFrame rows.

**Examples**

```javascript
df.shuffle()
```

Returns **[DataFrame](#dataframe)** A shuffled DataFrame.

### sample

[src/dataframe.js:808-817](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L808-L817 "Source code on GitHub")

Return a random sample of rows.

**Parameters**

-   `percentage` **[Number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** A percentage of the orignal DataFrame giving the sample size.

**Examples**

```javascript
df.sample(0.3)
```

Returns **[DataFrame](#dataframe)** A sample DataFrame

### bisect

[src/dataframe.js:826-840](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L826-L840 "Source code on GitHub")

Randomly split a DataFrame into 2 DataFrames.

**Parameters**

-   `percentage` **[Number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** A percentage of the orignal DataFrame giving the first DataFrame size. The second takes the rest.

**Examples**

```javascript
const [30DF, 70DF] = df.bisect(0.3)
```

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** An Array containing the two DataFrames. First, the X% DataFrame then the rest DataFrame.

### groupBy

[src/dataframe.js:853-855](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L853-L855 "Source code on GitHub")

Group DataFrame rows by columns giving a GroupedDataFrame object. See its doc for more examples.

**Parameters**

-   `columnNames` **...[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The columns used for the groupBy.

**Examples**

```javascript
df.groupBy('column1')
df.groupBy('column1', 'column2')
df.groupBy('column1', 'column2').listGroups()
df.groupBy('column1', 'column2').show()
df.groupBy('column1', 'column2').aggregate((group) => group.count())
```

Returns **GroupedDataFrame** A GroupedDataFrame object.

### sortBy

[src/dataframe.js:865-872](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L865-L872 "Source code on GitHub")

Sort DataFrame rows based on a column values. The row should contains only one variable type.

**Parameters**

-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column giving order.
-   `reverse` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Reverse mode. Reverse the order if true. (optional, default `false`)

**Examples**

```javascript
df.sortBy('id')
```

Returns **[DataFrame](#dataframe)** An ordered DataFrame.

### union

[src/dataframe.js:881-887](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L881-L887 "Source code on GitHub")

Concat two DataFrames.

**Parameters**

-   `dfToUnion` **[DataFrame](#dataframe)** The DataFrame to concat.

**Examples**

```javascript
df.union(df2)
```

Returns **[DataFrame](#dataframe)** A new concatenated DataFrame resulting of the union.

### join

[src/dataframe.js:898-907](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L898-L907 "Source code on GitHub")

Join two DataFrames.

**Parameters**

-   `dfToJoin` **[DataFrame](#dataframe)** The DataFrame to join.
-   `columnNames` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array))** The selected columns for the join.
-   `how` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The join mode. Can be: full, inner, outer, left, right. (optional, default `'inner'`)

**Examples**

```javascript
df.join(df2, 'column1', 'full')
```

Returns **[DataFrame](#dataframe)** The joined DataFrame.

### innerJoin

[src/dataframe.js:919-921](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L919-L921 "Source code on GitHub")

Join two DataFrames with inner mode.

**Parameters**

-   `dfToJoin` **[DataFrame](#dataframe)** The DataFrame to join.
-   `columnNames` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array))** The selected columns for the join.

**Examples**

```javascript
df.innerJoin(df2, 'id')
df.join(df2, 'id')
df.join(df2, 'id', 'inner')
```

Returns **[DataFrame](#dataframe)** The joined DataFrame.

### fullJoin

[src/dataframe.js:932-934](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L932-L934 "Source code on GitHub")

Join two DataFrames with full mode.

**Parameters**

-   `dfToJoin` **[DataFrame](#dataframe)** The DataFrame to join.
-   `columnNames` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array))** The selected columns for the join.

**Examples**

```javascript
df.fullJoin(df2, 'id')
df.join(df2, 'id', 'full')
```

Returns **[DataFrame](#dataframe)** The joined DataFrame.

### outerJoin

[src/dataframe.js:945-947](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L945-L947 "Source code on GitHub")

Join two DataFrames with outer mode.

**Parameters**

-   `dfToJoin` **[DataFrame](#dataframe)** The DataFrame to join.
-   `columnNames` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array))** The selected columns for the join.

**Examples**

```javascript
df2.outerJoin(df2, 'id')
df2.join(df2, 'id', 'outer')
```

Returns **[DataFrame](#dataframe)** The joined DataFrame.

### leftJoin

[src/dataframe.js:958-960](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L958-L960 "Source code on GitHub")

Join two DataFrames with left mode.

**Parameters**

-   `dfToJoin` **[DataFrame](#dataframe)** The DataFrame to join.
-   `columnNames` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array))** The selected columns for the join.

**Examples**

```javascript
df.leftJoin(df2, 'id')
df.join(df2, 'id', 'left')
```

Returns **[DataFrame](#dataframe)** The joined DataFrame.

### rightJoin

[src/dataframe.js:971-973](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L971-L973 "Source code on GitHub")

Join two DataFrames with right mode.

**Parameters**

-   `dfToJoin` **[DataFrame](#dataframe)** The DataFrame to join.
-   `columnNames` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array))** The selected columns for the join.

**Examples**

```javascript
df.rightJoin(df2, 'id')
df.join(df2, 'id', 'right')
```

Returns **[DataFrame](#dataframe)** The joined DataFrame.

### diff

[src/dataframe.js:983-985](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L983-L985 "Source code on GitHub")

Find the differences between two DataFrames (reverse of join).

**Parameters**

-   `dfToDiff` **[DataFrame](#dataframe)** The DataFrame to diff.
-   `columnNames` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array))** The selected columns for the diff.

**Examples**

```javascript
df2.diff(df2, 'id')
```

Returns **[DataFrame](#dataframe)** The differences DataFrame.

### setDefaultModules

[src/dataframe.js:23-25](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L23-L25 "Source code on GitHub")

Set the default modules used in DataFrame instances.

**Parameters**

-   `defaultModules` **...[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** DataFrame modules used by default.

**Examples**

```javascript
DataFrame.setDefaultModules(SQL, Stat)
```

### fromDSV

[src/dataframe.js:40-58](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L40-L58 "Source code on GitHub")

Create a DataFrame from a delimiter separated values text file. It returns a Promise.

**Parameters**

-   `pathOrFile` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) | File)** A path to the file (url or local) or a browser File object.
-   `sep` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The separator used to parse the file. (optional, default `';'`)
-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** A boolean indicating if the text has a header or not. (optional, default `true`)

**Examples**

```javascript
DataFrame.fromDSV('http://myurl/myfile.txt').then(df => df.show())
// In browser Only
DataFrame.fromDSV(myFile).then(df => df.show())
// From node.js only Only
DataFrame.fromDSV('/my/absolue/path/myfile.txt').then(df => df.show())
DataFrame.fromDSV('/my/absolue/path/myfile.txt', ';', true).then(df => df.show())
```

### fromText

[src/dataframe.js:73-75](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L73-L75 "Source code on GitHub")

Create a DataFrame from a delimiter separated values text file. It returns a Promise. Alias of DataFrame.fromDSV.

**Parameters**

-   `pathOrFile` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) | File)** A path to the file (url or local) or a browser File object.
-   `sep` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The separator used to parse the file. (optional, default `';'`)
-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** A boolean indicating if the text has a header or not. (optional, default `true`)

**Examples**

```javascript
DataFrame.fromText('http://myurl/myfile.txt').then(df => df.show())
// In browser Only
DataFrame.fromText(myFile).then(df => df.show())
// From node.js only Only
DataFrame.fromText('/my/absolue/path/myfile.txt').then(df => df.show())
DataFrame.fromText('/my/absolue/path/myfile.txt', ';', true).then(df => df.show())
```

### fromCSV

[src/dataframe.js:89-91](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L89-L91 "Source code on GitHub")

Create a DataFrame from a comma separated values file. It returns a Promise.

**Parameters**

-   `pathOrFile` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) | File)** A path to the file (url or local) or a browser File object.
-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** A boolean indicating if the csv has a header or not. (optional, default `true`)

**Examples**

```javascript
DataFrame.fromCSV('http://myurl/myfile.csv').then(df => df.show())
// For browser only
DataFrame.fromCSV(myFile).then(df => df.show())
// From node.js only
DataFrame.fromCSV('/my/absolue/path/myfile.csv').then(df => df.show())
DataFrame.fromCSV('/my/absolue/path/myfile.csv', true).then(df => df.show())
```

### fromTSV

[src/dataframe.js:105-107](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L105-L107 "Source code on GitHub")

Create a DataFrame from a tab separated values file. It returns a Promise.

**Parameters**

-   `pathOrFile` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) | File)** A path to the file (url or local) or a browser File object.
-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** A boolean indicating if the tsv has a header or not. (optional, default `true`)

**Examples**

```javascript
DataFrame.fromTSV('http://myurl/myfile.tsv').then(df => df.show())
// For browser only
DataFrame.fromTSV(myFile).then(df => df.show())
// From node.js only
DataFrame.fromTSV('/my/absolue/path/myfile.tsv').then(df => df.show())
DataFrame.fromTSV('/my/absolue/path/myfile.tsv', true).then(df => df.show())
```

### fromPSV

[src/dataframe.js:121-123](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L121-L123 "Source code on GitHub")

Create a DataFrame from a pipe separated values file. It returns a Promise.

**Parameters**

-   `pathOrFile` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) | File)** A path to the file (url or local) or a browser File object.
-   `header` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** A boolean indicating if the psv has a header or not. (optional, default `true`)

**Examples**

```javascript
DataFrame.fromPSV('http://myurl/myfile.psv').then(df => df.show())
// For browser only
DataFrame.fromPSV(myFile).then(df => df.show())
// From node.js only
DataFrame.fromPSV('/my/absolue/path/myfile.psv').then(df => df.show())
DataFrame.fromPSV('/my/absolue/path/myfile.psv', true).then(df => df.show())
```

### fromJSON

[src/dataframe.js:135-147](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/dataframe.js#L135-L147 "Source code on GitHub")

Create a DataFrame from a JSON file. It returns a Promise.

**Parameters**

-   `pathOrFile` **([String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) | File)** A path to the file (url or local) or a browser File object.

**Examples**

```javascript
DataFrame.fromJSON('http://myurl/myfile.json').then(df => df.show())
// For browser only
DataFrame.fromJSON(myFile).then(df => df.show())
// From node.js only
DataFrame.fromJSON('/my/absolute/path/myfile.json').then(df => df.show())
```
