<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

### Table of Contents

-   [GroupedDataFrame](#groupeddataframe)
    -   [constructor](#constructor)
    -   [toCollection](#tocollection)
    -   [show](#show)
    -   [listGroups](#listgroups)
    -   [listHashs](#listhashs)
    -   [aggregate](#aggregate)
    -   [pivot](#pivot)
    -   [melt](#melt)

## GroupedDataFrame

[src/groupedDataframe.js:9-166](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L9-L166 "Source code on GitHub")

Grouped DataFrame structure grouping DataFrame rows by column value.

**Parameters**

-   `df`  
-   `columnNames` **...any** 

### constructor

[src/groupedDataframe.js:20-25](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L20-L25 "Source code on GitHub")

Create a GroupedDataFrame. Used in DataFrame.groupBy('columnName').

**Parameters**

-   `df` **DataFrame** The DataFrame to group by.
-   `columnNames` **...any** 
-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column used for the group by.

**Examples**

```javascript
df.groupBy('column1');
//or
new GroupedDataFrame(df, 'column1');
```

### toCollection

[src/groupedDataframe.js:59-61](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L59-L61 "Source code on GitHub")

Convert GroupedDataFrame into collection (Array) of dictionnaries (Object).

**Examples**

```javascript
groupedDF.toCollection();
```

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** An Array containing group: {groupKey, group}.

### show

[src/groupedDataframe.js:70-78](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L70-L78 "Source code on GitHub")

Display the GroupedDataFrame as String Table.

**Parameters**

-   `quiet` **[Boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** Quiet mode. If true, it doesn't trigger console.log(). (optional, default `false`)

**Examples**

```javascript
groupedDf.show()
```

Returns **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The GroupedDataFrame as String Table.

### listGroups

[src/groupedDataframe.js:86-88](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L86-L88 "Source code on GitHub")

List GroupedDataFrame groups.

**Examples**

```javascript
gdf.listGroups()
```

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** An Array containing GroupedDataFrame group names.

### listHashs

[src/groupedDataframe.js:96-98](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L96-L98 "Source code on GitHub")

List GroupedDataFrame groups as a hashCode.

**Examples**

```javascript
gdf.listHashCodes()
```

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)** An Array containing GroupedDataFrame hash codes.

### aggregate

[src/groupedDataframe.js:108-113](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L108-L113 "Source code on GitHub")

Create an aggregation from a function.

**Parameters**

-   `func` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** The aggregation function.
-   `columnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column name created by the aggregation. (optional, default `'aggregation'`)

**Examples**

```javascript
groupedDF.aggregate(group => group.stat.sum('column1'));
```

Returns **DataFrame** A new DataFrame with a column 'aggregation' containing the result.

### pivot

[src/groupedDataframe.js:123-133](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L123-L133 "Source code on GitHub")

Pivot a GroupedDataFrame.

**Parameters**

-   `columnToPivot` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column which will be transposed as columns.
-   `func` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** The function to define each column value from a DataFrame. (optional, default `(gdf)`)

**Examples**

```javascript
df.groupBy('carType').pivot('carModel', values => values.stat.sum('kms'))
```

Returns **DataFrame** The pivot DataFrame.

### melt

[src/groupedDataframe.js:143-165](https://github.com/Gmousse/dataframe-js/blob/8a9b7e074e26a88e1026451b85d965049ba49d6b/src/groupedDataframe.js#L143-L165 "Source code on GitHub")

Melt a DataFrame to make it tidy. It's the reverse of GroupedDataFrame.pivot().

**Parameters**

-   `variableColumnName` **[String](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The column name containing values. (optional, default `'value'`)
-   `valueColumnName`   (optional, default `'value'`)

**Examples**

```javascript
df.groupBy('carType').melt('kms')
```

Returns **DataFrame** The tidy DataFrame.
